# MIT License
#
# Copyright (c) 2018 Evgeny Medvedev, evge.medvedev@gmail.com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.



# https://github.com/maticnetwork/launch/blob/master/mainnet-v1/sentry/sentry/bor/genesis.json

MAINNET_GENESIS_ALLOC = [
    # to_address, value
    ("0x0000000000000000000000000000000000001000", 0),
    ("0x0000000000000000000000000000000000001001", 0),
    ("0x0000000000000000000000000000000000001010", 9999993000000000000000000000),
    ("0x5973918275c01f50555d44e92c9d9b353cadad54", 1000000000000000000000),
    ("0xb8bb158b93c94ed35c1970d610d1e2b34e26652c", 1000000000000000000000),
    ("0xf84c74dea96df0ec22e11e7c33996c73fcc2d822", 1000000000000000000000),
    ("0xb702f1c9154ac9c08da247a8e30ee6f2f3373f41", 1000000000000000000000),
    ("0x7fcd58c2d53d980b247f1612fdba93e9a76193e6", 1000000000000000000000),
    ("0x0375b2fc7140977c9c76d45421564e354ed42277", 1000000000000000000000),
    ("0x42eefcda06ead475cde3731b8eb138e88cd0bac3", 1000000000000000000000)
]

