# MIT License
#
# Copyright (c) 2018 Evgeny Medvedev, evge.medvedev@gmail.com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


import json

from tests.polygonetl.job.mock_web3_provider import MockWeb3Provider, build_file_name


class MockBatchWeb3Provider(MockWeb3Provider):
    def __init__(self, read_resource):
        super().__init__(read_resource)
        self.read_resource = read_resource

    def make_batch_request(self, text):
        batch = json.loads(text)
        web3_response = []
        if isinstance(batch, dict):
            method = batch["method"]
            params = batch["params"]
            file_name = build_file_name(method, params)
            file_content = self.read_resource(file_name)
            return json.loads(file_content)
        else:
            for req in batch:
                method = req["method"]
                params = req["params"]
                file_name = build_file_name(method, params)
                file_content = self.read_resource(file_name)
                web3_response.append(json.loads(file_content))
        return web3_response
